  /*
   Problema: 
   Dato n, stampare la lista di tutti i sottoinsiemi di un dato universo U={1,...,n}
   Ad es. se n=3 e U={1,2,3}
   Sottoinsiemi: {},{1},{2},{3},{1,2},{2,3},...
   Quanti sono? 
   Un numero esponenziale 2 elevato alla n             
   Il mio algoritmo dovra fare almeno tanti passi quanti sono i sottoinsiemi...   
   
   Come rappresento un sottoinsieme S di {1,...,n}? 
   Es. per n=6 come rappresento S={1,3,4}?
           
   Idea: 
   Rappresento un insieme T tramite la sua funzione caratteristica
   
   f_T: {1,...,n} -> {0,1}, f(x)=1 sse x sta in S
   
   Es.  f_S(x)=1 per x=1,3,4 e =0 per gli altri numeri in 1...6
     
   
   2)  Ora vediamo come calcolare i sottoinsiemi
   
   Idea: 
   a) funzione caratteristica = numero binario su n cifre!!!
   b) generare numeri binari su n cifre = uso la somma bit a bit
   
   Sottoinsiemi di {1,...,n} = Tutti i numeri binari si n cifre!
   
   0 0 0 ... 0 -->  insieme vuoto
   1 0 0 ... 0 --> {1}
   0 1 0 ... 0 --> {2}
   1 1 0 ... 0 --> {1,2}
   ....
   
   Come li genero? 
   Possiamo usare un alg. semplice: 
   
   Partiamo da B=0... 0
   Per 2^n volte: 
       Sommo 1 a B
       stampo l'insieme che corrisponde al numero B       

   Mi serve: 
             - dichiarazione del vettore B di dim. Max
             - lettura del numero N
             - inizializzazione del vettore B
             - stampa dell'insieme associato a B
             - ciclo di calcolo della somma +1 su B

   */  
   


#include <stdio.h>
#define Max 10
// Calcolare tutti i sottoinsiemi di {1,...,n}

// strutture dati globali
int ok,N=0;
int subset[Max]; 
  // rapp. binaria di un sottoinsieme in {1,...N}

//Prototipi

// Funzione di lettura
void leggi_numero(int *N, int M);

// Funzione di inizializzazione
void inizializza_vettore(int N);

// Funzione di scrittura
void stampa_sottoinsieme(int N);

// Funzione di elaborazione

void calcola_sottoinsiemi(int N);

// Utilities
void prosegui();

int main(){

// Lettura 

leggi_numero(&N,Max);

prosegui();

// Inizializzazione Vettore

inizializza_vettore(N);

// Elaborazione 

calcola_sottoinsiemi(N);

return 0;
}

void leggi_numero(int *N, int M){
 do {
   printf("Inserisci tra 1 e %d: ",M);
   scanf("%d",N);
 }
 while (*N<=0 || *N>M);    
 
 printf("Valore letto: %d",*N);
}


void inizializza_vettore(N){
   int i;
   for (i=0;i<N;i++){ 
       subset[i]=0;
   } 
   subset[N]=0; // per la terminazione!   
}
 
void calcola_sottoinsiemi(N){
 int k;
 while (!subset[N]){
   int i=0;
   while(subset[i])  subset[i++]=0;  
   subset[i]=1;
   stampa_sottoinsieme(N);
 }
 prosegui();
}

void stampa_sottoinsieme(N){
  int i;
  for (i=0;i<N;i++)
    if (subset[i]==1) printf("%d ",i+1);   
  printf("\n");    
}


// Funzione di debug

void prosegui(){
 printf("\nVuoi proseguire? [1=si/0=no]: ");
 scanf("%d",&ok);
 if (!ok) exit(0);
}
